! decay.f90
! Compute decay from binmerge8.bin file.
! Miroslav Broz (miroslav.broz@email.cz), Jul 31st 2021

module swift

contains

integer function io_read_hdr_r8(iu,time,nbod,nleft) 

implicit none
integer, intent(in) :: iu
integer, intent(out) :: nbod,nleft
real(8), intent(out) :: time

real(8) :: ttmp
integer(2) :: nleft2,nbod2
integer :: ierr
character(len=128) :: msg

read(iu, iostat=ierr, iomsg=msg) ttmp,nbod2,nleft2
io_read_hdr_r8 = ierr
if (ierr.ne.0) then
  write(*,*) msg
  return
endif

nbod = nbod2
nleft = nleft2
time = ttmp

return
end function io_read_hdr_r8

integer function io_read_line_r8(iu,id,a,e,inc,capom,omega,capm) 

integer, intent(in) :: iu
integer, intent(out) :: id
real(8), intent(out) :: a,e,inc,capom,omega,capm

integer(2) :: id2
real(8) :: a8,e8,inc8,capom8,omega8,capm8
integer :: ierr

read(iu, iostat=ierr) id2,a8,e8,inc8,capom8,omega8,capm8
io_read_line_r = ierr
if (ierr.ne.0) then
  return
endif

id = id2
a = a8
e = e8
inc = inc8
capom = capom8
capm = capm8
omega = omega8

return
end function io_read_line_r8

end module swift

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

program decay

use swift

implicit none
integer :: i,j,maxj
integer :: iu,nbod,ntp,id,ierr
real(8) :: time,a,e,inc,node,peri,manom
real(8) :: dt,last
integer, dimension(1000000) :: num

dt = 1000.d0*365.25d0
num = 0
maxj = 0

iu = 1
open(unit=iu, file="binmerge8.bin", status="old", form='unformatted', iostat=ierr)
if (ierr.ne.0) then
  write(0,*) 'decay: Error opening file.', ierr
  stop
endif

do while (ierr.eq.0)
  ierr = io_read_hdr_r8(iu,time,nbod,ntp)
  if (ierr.ne.0) then
    write(0,*) 'decay: Error reading time.', ierr
    exit
  endif

  do i = 2, nbod
    ierr = io_read_line_r8(iu,id,a,e,inc,node,peri,manom)
    if (ierr.ne.0) then
      write(0,*) 'decay: Error reading PLs.'
      exit
    endif
  enddo

  do i = 1, ntp
    ierr = io_read_line_r8(iu,id,a,e,inc,node,peri,manom)
    if (ierr.ne.0) then
      write(0,*) 'decay: Error reading TPs.'
      exit
    endif
  enddo

  ! which time?
  j = int(time/dt)+1
  maxj = max(j,maxj)
  if (j.gt.size(num)) then
    write(0,*) 'decay: Error j > ', size(num), '.'
    stop
  endif
  num(j) = num(j)+ntp

enddo

close(iu)

! check jumps
j = 1
do while (num(j).eq.0)
  j = j+1
enddo
last = num(j)
do j = 1,maxj
!  if ((num(j).lt.last-50).or.(num(j).gt.last+50)) then
!    num(j) = last
!  endif
  last = num(j)
enddo

! output
open(unit=iu, file="decay.out", status="unknown")

write(iu,*) '# time num'
write(iu,*) '# d 1'
do j = 1,maxj
  if (num(j).gt.0) then
    time = (j-1)*dt
    write(iu,*) time, num(j)
  endif
enddo

close(iu)

stop

end program decay


